function [c1,saveC] = kmeans(x,nc,rerun);
% KMEANS : k-means clustering
% c = kmeans(x,nc)
%	x       - d*n samples
%	nc      - number of clusters wanted
%	c       - calculated membership vector
% algorithm taken from Sing-Tze Bow, 'Pattern Recognition'

% Copyright (c) 1995 Frank Dellaert
% All rights Reserved

[d,n] = size(x);
bestDis = 10000000;
for mk=1:rerun,
%------------------------------------------------------------------------
% step 1: Arbitrarily choose nc samples as the initial cluster centers
%------------------------------------------------------------------------
z= Inf*ones(d,nc);
oldz = z;
for i=1:nc,
  % find a random sample not yet in z
  distance = 0;
  while (distance==0)
    j = fix(rand*n)+1;
    candidate = x(:,j);
    [dummy,dummy,distance] = nearest(z,candidate);
    if(sum(candidate > 90) >= 1)
     distance = 0;
    end
  end
  % assign it as the center of cluster i
  z(:,i) = candidate
end
z(z>90) = 0;
c = zeros(1,n);
D = zeros(nc,n);

while(1),
%------------------------------------------------------------------------
% step 2: distribute the patterns samples x to the chosen cluster domains
%         based on which cluster center is nearest
%------------------------------------------------------------------------
for j=1:nc,			% for every cluster
  center = z(:,j);		% get cluster center
  if (center ~= oldz(:,j)),	% has it moved ? 
    D(j,:) = sqrDist(x,center);	% calculate the sqr distances from x to center
  end
end
oldz = z;

% find minimum
[Dmin,index] = min(D);
curDis = sum(Dmin);
moved = sum(index~=c);
fprintf(2,'moved = %d\n',moved);
c = index;

%------------------------------------------------------------------------
% step 3: Update the cluster centers
%         i.e. calculate the mean and minimize critsse for each cluster
%------------------------------------------------------------------------
[nr,z,v] = clusterstats(x,c,nc);
z
%------------------------------------------------------------------------
% step 4: Check convergence
%------------------------------------------------------------------------
if (moved==0), break, end

%------------------------------------------------------------------------
end 
% while(1)
if curDis < bestDis 
  c1 = c;
  saveC = z;
  bestDis = curDis; 
end
end 
% for mk












